/**
 * Launcher.java
 * Copyright (C) 2011  Kevin Wang and Shan Shi
 * 
 * This file is part of Minecarft.
 * 
 * Minecarft is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Minecarft is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Minecarft.  If not, see <http://www.gnu.org/licenses/>.
 */

package minecarft;

import java.io.File;
import java.util.Arrays;
import java.util.Comparator;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import org.lwjgl.LWJGLException;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;

/**
 *
 * @author kevin
 */
public class Launcher extends javax.swing.JFrame {
    private static Launcher instance;
    
    public static Launcher getInstance() {
        if (instance == null) {
            instance = new Launcher();
        }
        return instance;
    }

    /** Creates new form Launcher */
    private Launcher() {
        initComponents();
        
        DisplayMode[] modes;
        try {
            modes = Display.getAvailableDisplayModes();
            Arrays.sort(modes, new Comparator<DisplayMode>() {

                public int compare(DisplayMode o1, DisplayMode o2) {
                    return (o2.getWidth() * o2.getHeight() + o2.getBitsPerPixel() + o2.getFrequency())
                            - (o1.getWidth() * o1.getHeight() + o1.getBitsPerPixel() + o1.getFrequency());
                }
            });
        } catch (LWJGLException e) {
            modes = new DisplayMode[0];
        }
        for (int i = 0; i < modes.length; i++) {
            displaySizeComboBox.addItem(modes[i]);
        }
    }
    
    public void incrementProgressBar() {
        progressBar.setValue(progressBar.getValue() + 1);
    }
    
    public void setProgressLabel(String str) {
        progressLabel.setText(str);
    }
    
    public void close() {
        dispose();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jFileChooser1 = new javax.swing.JFileChooser();
        displaySizeLabel = new javax.swing.JLabel();
        displaySizeComboBox = new javax.swing.JComboBox();
        fullscreenModeCheckBox = new javax.swing.JCheckBox();
        startMinecarftButton = new javax.swing.JButton();
        progressLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        jSeparator1 = new javax.swing.JSeparator();
        loadButton = new javax.swing.JButton();

        jFileChooser1.setDialogTitle("Load Savefile");
        jFileChooser1.setFileFilter(new SaveFilter());

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Minecarft Launcher");
        setResizable(false);

        displaySizeLabel.setText("Display size");

        fullscreenModeCheckBox.setText("Fullscreen mode");

        startMinecarftButton.setText("Start Minecarft");
        startMinecarftButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                startMinecarftButtonMouseReleased(evt);
            }
        });

        progressLabel.setText("Loading...");

        progressBar.setMaximum(26);

        loadButton.setText("Load Savefile");
        loadButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                loadButtonMouseReleased(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(displaySizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 211, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(displaySizeLabel)
                    .addComponent(fullscreenModeCheckBox)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(startMinecarftButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 156, Short.MAX_VALUE)
                        .addComponent(loadButton))
                    .addComponent(jSeparator1, javax.swing.GroupLayout.DEFAULT_SIZE, 376, Short.MAX_VALUE)
                    .addComponent(progressLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.DEFAULT_SIZE, 376, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(displaySizeLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(displaySizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(fullscreenModeCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(startMinecarftButton)
                    .addComponent(loadButton))
                .addGap(18, 18, 18)
                .addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(progressLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void startMinecarftButtonMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_startMinecarftButtonMouseReleased
        if (startMinecarftButton.isEnabled()) {
            startMinecarft(null);
        }
    }//GEN-LAST:event_startMinecarftButtonMouseReleased

    private void loadButtonMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_loadButtonMouseReleased
        int returnVal = jFileChooser1.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            startMinecarft(jFileChooser1.getSelectedFile());
        }
    }//GEN-LAST:event_loadButtonMouseReleased

    private class SaveFilter extends FileFilter {

        @Override
        public boolean accept(File file) {
            if (file.isDirectory()) {
                return true;
            }
            String filename = file.getName();
            int i = filename.lastIndexOf(".");
            if (i > 0 && i < filename.length() - 1) {
                return filename.substring(i + 1).toLowerCase().equals("sav");
            }
            return false;
        }

        @Override
        public String getDescription() {
            return "Minecarft Savefiles";
        }
    }
    
    private void startMinecarft(final File savefile) {
        displaySizeComboBox.setEnabled(false);
        fullscreenModeCheckBox.setEnabled(false);
        startMinecarftButton.setEnabled(false);
        loadButton.setEnabled(false);
        Thread thread;
        if (savefile != null) {
            thread = new Thread(new Runnable() {

                public void run() {
                    Minecarft m = new Minecarft((DisplayMode) displaySizeComboBox.getSelectedItem(), fullscreenModeCheckBox.isSelected(), savefile);
                    m.start();
                }
            });
        } else {
            thread = new Thread(new Runnable() {
                
                public void run() {
                    Minecarft m = new Minecarft((DisplayMode) displaySizeComboBox.getSelectedItem(), fullscreenModeCheckBox.isSelected());
                    m.start();
                }
            });
        }
        thread.start();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox displaySizeComboBox;
    private javax.swing.JLabel displaySizeLabel;
    private javax.swing.JCheckBox fullscreenModeCheckBox;
    private javax.swing.JFileChooser jFileChooser1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JButton loadButton;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel progressLabel;
    private javax.swing.JButton startMinecarftButton;
    // End of variables declaration//GEN-END:variables
}
